@@ -1,6 +1,4 @@
 # list
-mrs = gl.project_mergerequests.list(project_id=1)
-# or
 mrs = project.mergerequests.list()
 # end list
 
@@ -9,17 +7,10 @@ mrs = project.mergerequests.list(state='merged', order_by='updated_at')
 # end filtered list
 
 # get
-mr = gl.project_mergerequests.get(mr_id, project_id=1)
-# or
 mr = project.mergerequests.get(mr_id)
 # end get
 
 # create
-mr = gl.project_mergerequests.create({'source_branch': 'cool_feature',
-                                      'target_branch': 'master',
-                                      'title': 'merge cool feature'},
-                                     project_id=1)
-# or
 mr = project.mergerequests.create({'source_branch': 'cool_feature',
                                    'target_branch': 'master',
                                    'title': 'merge cool feature'})
@@ -36,8 +27,6 @@ mr.save()
 # end state
 
 # delete
-gl.project_mergerequests.delete(mr_id, project_id=1)
-# or
 project.mergerequests.delete(mr_id)
 # or
 mr.delete()
@@ -48,7 +37,8 @@ mr.merge()
 # end merge
 
 # cancel
-mr.cancel_merge_when_build_succeeds()
+mr.cancel_merge_when_build_succeeds()  # v3
+mr.cancel_merge_when_pipeline_succeeds()  # v4
 # end cancel
 
 # issues
