@@ -34,7 +34,7 @@ from gitlab.exceptions import *  # noqa
 from gitlab.v3.objects import *  # noqa
 
 __title__ = 'python-gitlab'
-__version__ = '1.0.0'
+__version__ = '1.0.1'
 __author__ = 'Gauvain Pocentek'
 __email__ = 'gauvain@pocentek.net'
 __license__ = 'LGPL3'
@@ -193,15 +193,16 @@ class Gitlab(object):
         if not self.email or not self.password:
             raise GitlabAuthenticationError("Missing email/password")
 
+        data = {'email': self.email, 'password': self.password}
         if self.api_version == '3':
-            data = json.dumps({'email': self.email, 'password': self.password})
-            r = self._raw_post('/session', data,
+            r = self._raw_post('/session', json.dumps(data),
                                content_type='application/json')
             raise_error_from_response(r, GitlabAuthenticationError, 201)
             self.user = self._objects.CurrentUser(self, r.json())
         else:
-            manager = self._objects.CurrentUserManager()
-            self.user = manager.get(self.email, self.password)
+            r = self.http_post('/session', data)
+            manager = self._objects.CurrentUserManager(self)
+            self.user = self._objects.CurrentUser(manager, r)
 
         self._set_token(self.user.private_token)
 
@@ -396,11 +397,13 @@ class Gitlab(object):
 
         return results
 
-    def _raw_post(self, path_, data=None, content_type=None, **kwargs):
+    def _raw_post(self, path_, data=None, content_type=None,
+                  files=None, **kwargs):
         url = '%s%s' % (self._url, path_)
         opts = self._get_session_opts(content_type)
         try:
-            return self.session.post(url, params=kwargs, data=data, **opts)
+            return self.session.post(url, params=kwargs, data=data,
+                                     files=files, **opts)
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
@@ -628,7 +631,7 @@ class Gitlab(object):
             return '%s%s' % (self._url, path)
 
     def http_request(self, verb, path, query_data={}, post_data={},
-                     streamed=False, **kwargs):
+                     streamed=False, files=None, **kwargs):
         """Make an HTTP request to the Gitlab server.
 
         Args:
@@ -658,6 +661,11 @@ class Gitlab(object):
         params = query_data.copy()
         params.update(kwargs)
         opts = self._get_session_opts(content_type='application/json')
+
+        # don't set the content-type header when uploading files
+        if files is not None:
+            del opts["headers"]["Content-type"]
+
         verify = opts.pop('verify')
         timeout = opts.pop('timeout')
 
@@ -668,7 +676,7 @@ class Gitlab(object):
         # always agree with this decision (this is the case with a default
         # gitlab installation)
         req = requests.Request(verb, url, json=post_data, params=params,
-                               **opts)
+                               files=files, **opts)
         prepped = self.session.prepare_request(req)
         prepped.url = sanitized_url(prepped.url)
         result = self.session.send(prepped, stream=streamed, verify=verify,
@@ -677,12 +685,19 @@ class Gitlab(object):
         if 200 <= result.status_code < 300:
             return result
 
+        try:
+            error_message = result.json()['message']
+        except (KeyError, ValueError, TypeError):
+            error_message = result.content
+
         if result.status_code == 401:
             raise GitlabAuthenticationError(response_code=result.status_code,
-                                            error_message=result.content)
+                                            error_message=error_message,
+                                            response_body=result.content)
 
         raise GitlabHttpError(response_code=result.status_code,
-                              error_message=result.content)
+                              error_message=error_message,
+                              response_body=result.content)
 
     def http_get(self, path, query_data={}, streamed=False, **kwargs):
         """Make a GET request to the Gitlab server.
@@ -754,7 +769,8 @@ class Gitlab(object):
         # No pagination, generator requested
         return GitlabList(self, url, query_data, **kwargs)
 
-    def http_post(self, path, query_data={}, post_data={}, **kwargs):
+    def http_post(self, path, query_data={}, post_data={}, files=None,
+                  **kwargs):
         """Make a POST request to the Gitlab server.
 
         Args:
@@ -774,7 +790,7 @@ class Gitlab(object):
             GitlabParsingError: If the json data could not be parsed
         """
         result = self.http_request('post', path, query_data=query_data,
-                                   post_data=post_data, **kwargs)
+                                   post_data=post_data, files=files, **kwargs)
         try:
             if result.headers.get('Content-Type', None) == 'application/json':
                 return result.json()
