@@ -689,7 +689,7 @@ class ProjectJob(RESTObject):
             str: The artifacts if `streamed` is False, None otherwise.
         """
         path = '%s/%s/artifacts' % (self.manager.path, self.get_id())
-        result = self.manager.gitlab.get_http(path, streamed=streamed,
+        result = self.manager.gitlab.http_get(path, streamed=streamed,
                                               **kwargs)
         return utils.response_content(result, streamed, action, chunk_size)
 
@@ -715,7 +715,7 @@ class ProjectJob(RESTObject):
             str: The trace
         """
         path = '%s/%s/trace' % (self.manager.path, self.get_id())
-        result = self.manager.gitlab.get_http(path, streamed=streamed,
+        result = self.manager.gitlab.http_get(path, streamed=streamed,
                                               **kwargs)
         return utils.response_content(result, streamed, action, chunk_size)
 
@@ -735,7 +735,7 @@ class ProjectCommitStatusManager(GetFromListMixin, CreateMixin, RESTManager):
              '/statuses')
     _obj_cls = ProjectCommitStatus
     _from_parent_attrs = {'project_id': 'project_id', 'commit_id': 'id'}
-    _create_attrs = (('state', 'sha'),
+    _create_attrs = (('state', ),
                      ('description', 'name', 'context', 'ref', 'target_url',
                       'coverage'))
 
@@ -960,6 +960,12 @@ class ProjectIssueManager(CRUDMixin, RESTManager):
                                'milestone_id', 'labels', 'created_at',
                                'updated_at', 'state_event', 'due_date'))
 
+    def _sanitize_data(self, data, action):
+        new_data = data.copy()
+        if 'labels' in data:
+            new_data['labels'] = ','.join(data['labels'])
+        return new_data
+
 
 class ProjectMember(SaveMixin, ObjectDeleteMixin, RESTObject):
     _short_print_attr = 'username'
@@ -1035,8 +1041,7 @@ class ProjectTag(ObjectDeleteMixin, RESTObject):
         self.release = server_data
 
 
-class ProjectTagManager(GetFromListMixin, CreateMixin, DeleteMixin,
-                        RESTManager):
+class ProjectTagManager(NoUpdateMixin, RESTManager):
     _path = '/projects/%(project_id)s/repository/tags'
     _obj_cls = ProjectTag
     _from_parent_attrs = {'project_id': 'id'}
@@ -1810,8 +1815,8 @@ class Project(SaveMixin, ObjectDeleteMixin, RESTObject):
             query_data['path'] = path
         if ref:
             query_data['ref'] = ref
-        return self.manager.gitlab.http_get(gl_path, query_data=query_data,
-                                            **kwargs)
+        return self.manager.gitlab.http_list(gl_path, query_data=query_data,
+                                             **kwargs)
 
     @cli.register_custom_action('Project', ('sha', ))
     @exc.on_http_error(exc.GitlabGetError)
@@ -1899,7 +1904,7 @@ class Project(SaveMixin, ObjectDeleteMixin, RESTObject):
             list: The contributors
         """
         path = '/projects/%s/repository/contributors' % self.get_id()
-        return self.manager.gitlab.http_get(path, **kwargs)
+        return self.manager.gitlab.http_list(path, **kwargs)
 
     @cli.register_custom_action('Project', tuple(), ('sha', ))
     @exc.on_http_error(exc.GitlabListError)
@@ -2072,6 +2077,59 @@ class Project(SaveMixin, ObjectDeleteMixin, RESTObject):
         post_data.update(form)
         self.manager.gitlab.http_post(path, post_data=post_data, **kwargs)
 
+    # see #56 - add file attachment features
+    @cli.register_custom_action('Project', ('filename', 'filepath'))
+    @exc.on_http_error(exc.GitlabUploadError)
+    def upload(self, filename, filedata=None, filepath=None, **kwargs):
+        """Upload the specified file into the project.
+
+        .. note::
+
+            Either ``filedata`` or ``filepath`` *MUST* be specified.
+
+        Args:
+            filename (str): The name of the file being uploaded
+            filedata (bytes): The raw data of the file being uploaded
+            filepath (str): The path to a local file to upload (optional)
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached
+            GitlabUploadError: If the file upload fails
+            GitlabUploadError: If ``filedata`` and ``filepath`` are not
+                specified
+            GitlabUploadError: If both ``filedata`` and ``filepath`` are
+                specified
+
+        Returns:
+            dict: A ``dict`` with the keys:
+                * ``alt`` - The alternate text for the upload
+                * ``url`` - The direct url to the uploaded file
+                * ``markdown`` - Markdown for the uploaded file
+        """
+        if filepath is None and filedata is None:
+            raise GitlabUploadError("No file contents or path specified")
+
+        if filedata is not None and filepath is not None:
+            raise GitlabUploadError("File contents and file path specified")
+
+        if filepath is not None:
+            with open(filepath, "rb") as f:
+                filedata = f.read()
+
+        url = ('/projects/%(id)s/uploads' % {
+            'id': self.id,
+        })
+        file_info = {
+            'file': (filename, filedata),
+        }
+        data = self.manager.gitlab.http_post(url, files=file_info)
+
+        return {
+            "alt": data['alt'],
+            "url": data['url'],
+            "markdown": data['markdown']
+        }
+
 
 class Runner(SaveMixin, ObjectDeleteMixin, RESTObject):
     pass
